# python-cc1101 - Python Library to Transmit RF Signals via CC1101 Transceivers
#
# Copyright (C) 2021 Fabian Peter Hammerle <fabian@hammerle.me>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import ctypes
import ctypes.util
import errno
import functools

# could not find Debian's python3-libgpiod on pypi.org
# https://salsa.debian.org/debian/libgpiod does not provide setup.py or setup.cfg


@functools.lru_cache(maxsize=1)
def _load_libgpiod() -> ctypes.CDLL:
    filename = ctypes.util.find_library("gpiod")
    if not filename:
        raise FileNotFoundError(
            "Failed to find libgpiod."
            "\nOn Debian-based systems, like Raspberry Pi OS / Raspbian,"
            " libgpiod can be installed via"
            "\n\tsudo apt-get install --no-install-recommends libgpiod2"
        )
    return ctypes.CDLL(filename, use_errno=True)


class _c_timespec(ctypes.Structure):
    """
    struct timespec {
        time_t tv_sec;
        long tv_nsec;
    };
    """

    # pylint: disable=too-few-public-methods,invalid-name; struct

    _fields_ = [("tv_sec", ctypes.c_long), ("tv_nsec", ctypes.c_long)]


class GPIOLine:
    def __init__(self, pointer: ctypes.c_void_p) -> None:
        assert pointer != 0
        self._pointer = pointer

    @classmethod
    def find(cls, name: bytes) -> "GPIOLine":
        # > If this routine succeeds, the user must manually close the GPIO chip
        # > owning this line to avoid memory leaks.
        pointer = _load_libgpiod().gpiod_line_find(name)  # type: int
        # > If the line could not be found, this functions sets errno to ENOENT.
        if pointer == 0:
            err = ctypes.get_errno()
            if err == errno.EACCES:
                # > [PermissionError] corresponds to errno EACCES and EPERM.
                raise PermissionError(
                    "Failed to access GPIO line {!r}.".format(name.decode())
                    + "\nVerify that the current user has read and write access for /dev/gpiochip*."
                    + "\nOn some systems, like Raspberry Pi OS / Raspbian,"
                    + "\n\tsudo usermod -a -G gpio $USER"
                    + "\nfollowed by a re-login grants sufficient permissions."
                )
            if err == errno.ENOENT:
                # > [FileNotFoundError] corresponds to errno ENOENT.
                # https://docs.python.org/3/library/exceptions.html#FileNotFoundError
                raise FileNotFoundError(
                    "GPIO line {!r} does not exist.".format(name.decode())
                    + "\nRun command `gpioinfo` to get a list of all available GPIO lines."
                )
            raise OSError(
                "Failed to open GPIO line {!r}: {}".format(
                    name.decode(),
                    errno.errorcode[err],
                )
            )
        return cls(pointer=ctypes.c_void_p(pointer))

    def __del__(self):
        # > Close a GPIO chip owning this line and release all resources.
        # > After this function returns, the line must no longer be used.
        if self._pointer:
            _load_libgpiod().gpiod_line_close_chip(self._pointer)
        # might make debugging easier in case someone calls __del__ twice
        self._pointer = None

    def wait_for_rising_edge(self, consumer: bytes, timeout_seconds: int) -> bool:
        """
        Return True, if an event occured; False on timeout.
        """
        if (
            _load_libgpiod().gpiod_line_request_rising_edge_events(
                self._pointer, consumer
            )
            != 0
        ):
            err = ctypes.get_errno()
            raise OSError(
                "Request for rising edge event notifications failed ({}).".format(
                    errno.errorcode[err]
                )
                + ("\nBlocked by another process?" if err == errno.EBUSY else "")
            )
        timeout = _c_timespec(timeout_seconds, 0)
        result = _load_libgpiod().gpiod_line_event_wait(
            self._pointer, ctypes.pointer(timeout)
        )  # type: int
        _load_libgpiod().gpiod_line_release(self._pointer)
        if result == -1:
            raise OSError("Failed to wait for rising edge event notification.")
        return result == 1



import unittest.mock

import pytest

import cc1101


@pytest.fixture(scope="function")
def transceiver():
    with unittest.mock.patch("spidev.SpiDev"):
        return cc1101.CC1101()


@pytest.fixture(scope="function")
def libgpiod_mock():
    mock = unittest.mock.MagicMock()
    with unittest.mock.patch("cc1101._gpio._load_libgpiod", return_value=mock):
        yield mock



# python-cc1101 - Python Library to Transmit RF Signals via CC1101 Transceivers
#
# Copyright (C) 2020 Fabian Peter Hammerle <fabian@hammerle.me>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import unittest.mock

import pytest

from cc1101.options import PacketLengthMode

# pylint: disable=protected-access


@pytest.mark.parametrize(
    ("pktctrl0_before", "pktctrl0_after"),
    (
        # unchanged
        (0b00000000, 0b00000000),
        (0b00010000, 0b00010000),
        (0b00010001, 0b00010001),
        (0b01000000, 0b01000000),
        (0b01000010, 0b01000010),
        (0b01110000, 0b01110000),
        (0b01110010, 0b01110010),
        # disabled
        (0b00010100, 0b00010000),
        (0b01000100, 0b01000000),
        (0b01000110, 0b01000010),
        (0b01110110, 0b01110010),
    ),
)
def test_disable_checksum(transceiver, pktctrl0_before, pktctrl0_after):
    xfer_mock = transceiver._spi.xfer
    xfer_mock.return_value = [15, 15]
    with unittest.mock.patch.object(
        transceiver, "_read_single_byte", return_value=pktctrl0_before
    ):
        transceiver.disable_checksum()
    xfer_mock.assert_called_once_with([0x08 | 0x40, pktctrl0_after])


@pytest.mark.parametrize(
    ("pktctrl0", "expected_mode"),
    (
        (0b00000000, PacketLengthMode.FIXED),
        (0b00000001, PacketLengthMode.VARIABLE),
        (0b01000100, PacketLengthMode.FIXED),
        (0b01000101, PacketLengthMode.VARIABLE),
    ),
)
def test_get_packet_length_mode(transceiver, pktctrl0, expected_mode):
    xfer_mock = transceiver._spi.xfer
    xfer_mock.return_value = [0, pktctrl0]
    assert transceiver.get_packet_length_mode() == expected_mode
    xfer_mock.assert_called_once_with([0x08 | 0x80, 0])


@pytest.mark.parametrize(
    ("pktctrl0_before", "pktctrl0_after", "mode"),
    (
        (0b00000000, 0b00000000, PacketLengthMode.FIXED),
        (0b00000001, 0b00000000, PacketLengthMode.FIXED),
        (0b00000001, 0b00000001, PacketLengthMode.VARIABLE),
        (0b00000010, 0b00000000, PacketLengthMode.FIXED),
        (0b00000010, 0b00000001, PacketLengthMode.VARIABLE),
        (0b01000100, 0b01000100, PacketLengthMode.FIXED),
        (0b01000100, 0b01000101, PacketLengthMode.VARIABLE),
        (0b01000101, 0b01000100, PacketLengthMode.FIXED),
        (0b01000101, 0b01000101, PacketLengthMode.VARIABLE),
    ),
)
def test_set_packet_length_mode(transceiver, pktctrl0_before, pktctrl0_after, mode):
    xfer_mock = transceiver._spi.xfer
    xfer_mock.return_value = [15, 15]
    with unittest.mock.patch.object(
        transceiver, "_read_single_byte", return_value=pktctrl0_before
    ):
        transceiver.set_packet_length_mode(mode)
    xfer_mock.assert_called_once_with([0x08 | 0x40, pktctrl0_after])



# python-cc1101 - Python Library to Transmit RF Signals via CC1101 Transceivers
#
# Copyright (C) 2021 Fabian Peter Hammerle <fabian@hammerle.me>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import ctypes
import ctypes.util
import errno
import re
import unittest.mock

import pytest

import cc1101._gpio

# pylint: disable=protected-access


def test__load_libgpiod():
    with unittest.mock.patch(
        "ctypes.util.find_library", return_value=ctypes.util.find_library("c")
    ) as find_library_mock:
        assert isinstance(cc1101._gpio._load_libgpiod(), ctypes.CDLL)
    find_library_mock.assert_called_once_with("gpiod")


@pytest.mark.parametrize("name", ("GPIO24", "GPIO25"))
def test_line_find_permission_denied(libgpiod_mock, name):
    libgpiod_mock.gpiod_line_find.return_value = 0
    ctypes.set_errno(errno.EACCES)
    with pytest.raises(
        PermissionError,
        match=r"^Failed to access GPIO line {!r}\.\n".format(re.escape(name)),
    ):
        cc1101._gpio.GPIOLine.find(name.encode())


@pytest.mark.parametrize("name", ("GPIO24", "GPIO25"))
def test_line_find_non_existing(libgpiod_mock, name):
    libgpiod_mock.gpiod_line_find.return_value = 0
    ctypes.set_errno(errno.ENOENT)
    with pytest.raises(
        FileNotFoundError,
        match=r"^GPIO line {!r} does not exist\.\n".format(re.escape(name)),
    ):
        cc1101._gpio.GPIOLine.find(name.encode())


@pytest.mark.parametrize("name", ("GPIO24", "GPIO25"))
def test_line_find_unknown_error(libgpiod_mock, name):
    libgpiod_mock.gpiod_line_find.return_value = 0
    ctypes.set_errno(errno.ENOANO)
    with pytest.raises(
        OSError,
        match=r"^Failed to open GPIO line {!r}: ENOANO$".format(re.escape(name)),
    ):
        cc1101._gpio.GPIOLine.find(name.encode())


def test_line_find(libgpiod_mock):
    libgpiod_mock.gpiod_line_find.return_value = 21
    line = cc1101._gpio.GPIOLine.find(b"GPIO24")
    libgpiod_mock.gpiod_line_find.assert_called_once_with(b"GPIO24")
    assert isinstance(line, cc1101._gpio.GPIOLine)
    assert line._pointer.value == 21


def test_line_release(libgpiod_mock):
    line = cc1101._gpio.GPIOLine(42)
    del line
    libgpiod_mock.gpiod_line_close_chip.assert_called_once_with(42)


@pytest.mark.parametrize("consumer", (b"CC1101 GDO0", b"test"))
@pytest.mark.parametrize("timeout_seconds", (21, 42))
@pytest.mark.parametrize("reached_timeout", (False, True))
def test_line_wait_for_rising_edge(
    libgpiod_mock, consumer: bytes, timeout_seconds: int, reached_timeout: bool
):
    pointer = ctypes.c_void_p(1234)
    line = cc1101._gpio.GPIOLine(pointer=pointer)
    libgpiod_mock.gpiod_line_request_rising_edge_events.return_value = 0
    libgpiod_mock.gpiod_line_event_wait.return_value = 0 if reached_timeout else 1
    event_occured = line.wait_for_rising_edge(
        consumer=consumer, timeout_seconds=timeout_seconds
    )
    assert event_occured is not reached_timeout
    libgpiod_mock.gpiod_line_request_rising_edge_events.assert_called_once_with(
        pointer, consumer
    )
    assert libgpiod_mock.gpiod_line_event_wait.call_count == 1
    wait_args, wait_kwargs = libgpiod_mock.gpiod_line_event_wait.call_args
    assert wait_args[0] == pointer
    assert wait_args[1].contents.tv_nsec == 0
    assert wait_args[1].contents.tv_sec == timeout_seconds
    assert not wait_args[2:]
    assert not wait_kwargs
    libgpiod_mock.gpiod_line_release.assert_called_once_with(pointer)


@pytest.mark.parametrize("consumer", (b"CC1101 GDO0",))
@pytest.mark.parametrize("timeout_seconds", (21,))
def test_line_wait_for_rising_edge_busy(
    libgpiod_mock, consumer: bytes, timeout_seconds: int
):
    pointer = ctypes.c_void_p(1234)
    line = cc1101._gpio.GPIOLine(pointer=pointer)
    libgpiod_mock.gpiod_line_request_rising_edge_events.return_value = -1
    ctypes.set_errno(errno.EBUSY)
    with pytest.raises(
        OSError,
        match=r"^Request for rising edge event notifications failed \(EBUSY\)."
        r"\nBlocked by another process\?$",
    ):
        line.wait_for_rising_edge(consumer=consumer, timeout_seconds=timeout_seconds)



# python-cc1101 - Python Library to Transmit RF Signals via CC1101 Transceivers
#
# Copyright (C) 2020 Fabian Peter Hammerle <fabian@hammerle.me>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import pytest

import cc1101


# pylint: disable=protected-access


@pytest.mark.parametrize(
    "marcstate",
    (
        cc1101.MainRadioControlStateMachineState.IDLE,
        cc1101.MainRadioControlStateMachineState.RX,
    ),
)
def test_get_main_radio_control_state_machine_state(transceiver, marcstate):
    transceiver._spi.xfer.return_value = [
        0b0000111,  # chip status "idle", but irrelevant for retrieval of marcstate
        marcstate.value,
    ]
    assert transceiver.get_main_radio_control_state_machine_state() == marcstate
    transceiver._spi.xfer.assert_called_once_with([0x35 | 0xC0, 0])
    assert transceiver.get_marc_state() == marcstate



# python-cc1101 - Python Library to Transmit RF Signals via CC1101 Transceivers
#
# Copyright (C) 2020 Fabian Peter Hammerle <fabian@hammerle.me>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import unittest.mock

import pytest

# pylint: disable=protected-access


@pytest.mark.parametrize(
    ("frend0", "setting_index"),
    [
        (0b01000, 0),
        (0b01001, 1),
        (0b01111, 7),
        (0b10000, 0),
        (0b10001, 1),
        (0b10111, 7),
        (0b10101, 5),
    ],
)
def test__get_power_amplifier_setting_index(transceiver, frend0, setting_index):
    transceiver._spi.xfer.return_value = [15, frend0]
    assert transceiver._get_power_amplifier_setting_index() == setting_index
    transceiver._spi.xfer.assert_called_once_with([0x22 | 0x80, 0])


@pytest.mark.parametrize(
    ("frend0_before", "frend0_after", "setting_index"),
    [
        (0b01000, 0b01000, 0),
        (0b01000, 0b01001, 1),
        (0b01000, 0b01111, 7),
        (0b10000, 0b10000, 0),
        (0b10000, 0b10001, 1),
        (0b10000, 0b10111, 7),
    ],
)
def test__set_power_amplifier_setting_index(
    transceiver, frend0_before, frend0_after, setting_index
):
    transceiver._spi.xfer.return_value = [15, 15]
    with unittest.mock.patch.object(
        transceiver, "_read_single_byte", return_value=frend0_before
    ):
        transceiver._set_power_amplifier_setting_index(setting_index)
    transceiver._spi.xfer.assert_called_once_with([0x22 | 0x40, frend0_after])


@pytest.mark.parametrize("setting_index", (-1, 8, 21))
def test__set_power_amplifier_setting_index_invalid(transceiver, setting_index):
    with pytest.raises(Exception):
        transceiver._set_power_amplifier_setting_index(setting_index)
    transceiver._spi.xfer.assert_not_called()



import unittest.mock

import pytest

import cc1101


@pytest.mark.parametrize(
    ("transceiver_str", "sync_word"),
    (
        (
            "CC1101(marcstate=idle, base_frequency=433.92MHz, "
            "symbol_rate=2.14kBaud, modulation_format=ASK_OOK, "
            "sync_mode=NO_PREAMBLE_AND_SYNC_WORD, packet_length=21B, output_power=(0,0xc0))",
            None,
        ),
        (
            "CC1101(marcstate=idle, base_frequency=433.92MHz, "
            "symbol_rate=2.14kBaud, modulation_format=ASK_OOK, "
            "sync_mode=TRANSMIT_16_MATCH_15_BITS, preamble_length=4B, sync_word=0x01ef, "
            "packet_length=21B, output_power=(0,0xc0))",
            b"\x01\xef",
        ),
    ),
)
def test___str___(transceiver_str, sync_word):
    transceiver = cc1101.CC1101()
    with unittest.mock.patch.object(
        transceiver,
        "get_main_radio_control_state_machine_state",
        return_value=cc1101.MainRadioControlStateMachineState.IDLE,
    ), unittest.mock.patch.object(
        transceiver, "get_base_frequency_hertz", return_value=433.92e6
    ), unittest.mock.patch.object(
        transceiver, "get_symbol_rate_baud", return_value=2142
    ), unittest.mock.patch.object(
        transceiver,
        "get_modulation_format",
        return_value=cc1101.ModulationFormat.ASK_OOK,
    ), unittest.mock.patch.object(
        transceiver,
        "get_sync_mode",
        return_value=cc1101.SyncMode.TRANSMIT_16_MATCH_15_BITS
        if sync_word
        else cc1101.SyncMode.NO_PREAMBLE_AND_SYNC_WORD,
    ), unittest.mock.patch.object(
        transceiver, "get_preamble_length_bytes", return_value=4
    ), unittest.mock.patch.object(
        transceiver, "get_sync_word", return_value=sync_word
    ), unittest.mock.patch.object(
        transceiver,
        "get_packet_length_mode",
        return_value=cc1101.PacketLengthMode.FIXED,
    ), unittest.mock.patch.object(
        transceiver, "get_packet_length_bytes", return_value=21
    ), unittest.mock.patch.object(
        transceiver, "get_output_power", return_value=(0, 0xC0)
    ):
        assert str(transceiver) == transceiver_str



# python-cc1101 - Python Library to Transmit RF Signals via CC1101 Transceivers
#
# Copyright (C) 2020 Fabian Peter Hammerle <fabian@hammerle.me>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import unittest.mock

import pytest

# pylint: disable=protected-access


def test__enable_receive_mode(transceiver):
    transceiver._spi.xfer.return_value = [15]
    transceiver._enable_receive_mode()
    transceiver._spi.xfer.assert_called_once_with([0x34 | 0x00])


@pytest.mark.parametrize("payload", [b"\0", b"\x12\x45\x56"])
def test__get_received_packet(transceiver, payload):
    fifo_buffer = list(payload) + [128, (1 << 7) | 42]
    with unittest.mock.patch.object(
        transceiver, "_read_status_register", return_value=len(fifo_buffer)
    ) as read_status_register, unittest.mock.patch.object(
        transceiver, "_read_burst", return_value=fifo_buffer
    ) as read_burst_mock:
        received_packet = transceiver._get_received_packet()
    read_status_register.assert_called_once_with(0x3B)
    read_burst_mock.assert_called_once_with(
        start_register=0x3F, length=len(fifo_buffer)
    )
    assert received_packet.payload == payload
    assert received_packet._rssi_index == 128
    assert received_packet.checksum_valid
    assert received_packet.link_quality_indicator == 42


@pytest.mark.parametrize("gdo0_gpio_line_name", (b"GPIO24", b"GPIO25"))
@pytest.mark.parametrize("reached_timeout", (True, False))
@pytest.mark.parametrize("timeout_seconds", (2,))
def test__wait_for_packet(
    transceiver, gdo0_gpio_line_name, timeout_seconds, reached_timeout
):
    line_mock = unittest.mock.MagicMock()
    line_mock.wait_for_rising_edge.return_value = not reached_timeout
    with unittest.mock.patch(
        "cc1101._gpio.GPIOLine.find", return_value=line_mock
    ) as find_line_mock, unittest.mock.patch.object(
        transceiver, "_get_received_packet", return_value="packet-dummy"
    ) as get_received_packet_mock, unittest.mock.patch.object(
        transceiver, "_enable_receive_mode"
    ) as enable_receive_mode_mock, unittest.mock.patch.object(
        transceiver, "_command_strobe"
    ) as command_strobe_mock:
        packet = transceiver._wait_for_packet(
            timeout_seconds=timeout_seconds,
            gdo0_gpio_line_name=gdo0_gpio_line_name,
        )
    find_line_mock.assert_called_once_with(name=gdo0_gpio_line_name)
    enable_receive_mode_mock.assert_called_once_with()
    line_mock.wait_for_rising_edge.assert_called_once_with(
        consumer=b"CC1101:GDO0", timeout_seconds=timeout_seconds
    )
    if reached_timeout:
        assert packet is None
        command_strobe_mock.assert_called_once_with(0x36)  # SIDLE
        get_received_packet_mock.assert_not_called()
    else:
        command_strobe_mock.assert_not_called()
        get_received_packet_mock.assert_called_once_with()
        assert packet == "packet-dummy"



# python-cc1101 - Python Library to Transmit RF Signals via CC1101 Transceivers
#
# Copyright (C) 2020 Fabian Peter Hammerle <fabian@hammerle.me>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import pytest

# pylint: disable=protected-access


@pytest.mark.parametrize(
    ("xfer_return_value", "sync_word"),
    [([64, 211, 145], b"\xd3\x91"), ([64, 0, 0], b"\0\0")],
)
def test_get_sync_word(transceiver, xfer_return_value, sync_word):
    transceiver._spi.xfer.return_value = xfer_return_value
    assert transceiver.get_sync_word() == sync_word
    transceiver._spi.xfer.assert_called_once_with([0x04 | 0xC0, 0, 0])


def test_set_sync_word(transceiver):
    transceiver._spi.xfer.return_value = [15, 15, 15]
    transceiver.set_sync_word(b"\x12\x34")
    transceiver._spi.xfer.assert_called_once_with([0x04 | 0x40, 0x12, 0x34])


@pytest.mark.parametrize("sync_word", [b"", b"\0", "\x12\x34\x56"])
def test_set_sync_word_invalid_length(transceiver, sync_word):
    with pytest.raises(ValueError, match=r"\bexpected two bytes\b"):
        transceiver.set_sync_word(sync_word)
